/*------------------------------park2.c--------------------------------*/

/* Written by Ken Palliser - NOV 1989 */

/* park2.c - parks heads on 1 or 2 drives */


#include <stdio.h>
#include <dos.h>


#define disk 0x13                   /* BIOS disk interrupt */
#define drive_parms_function 0x08   /* Get drive info function */
#define seek_function 0x0c          /* Seek to cylinder function */


union REGS regs;               /* prototype in dos.h */
unsigned int mask1 = 0x00c0;   /* 00000000 11000000 */
unsigned int mask2 = 0xff00;   /* 11111111 00000000 */
unsigned int mask3 = 0x0300;   /* 00000011 00000000 */

int drive = 0x80;              /* First HDD = 0x80, 2nd = 0x81 */
int no_of_drives;              /* No. of HDD's known to BIOS */
unsigned int max_track;        /* Max. track ( as per drive initialization
                                  parameters, it is not necessarily max.
																	 physical track) */ 
                                   										
/* Function prototypes */

void get_max_tracks(void);     
void park_head(void);
unsigned int cx_to_track(unsigned int cx_reg);
unsigned int track_to_cx(unsigned int track);


int main (void)
{

	get_max_tracks();        /* Get first HDD max track number */
	++max_track;             /* Increment by 1 */
	park_head();             /* Move heads to the end of data (+1) */
	
	if (no_of_drives == 2)   /* Second flywheel in box? */
	{
		++drive;               /* Increment drive count */
		get_max_tracks();
		++max_track;
		park_head();           /* Park this one also */
	}
	else
	 printf("\nOnly one hard drive installed .. \n");
 
	exit(0);
	
	}			/************* end main ***************/


/*-------------------- get_max_tracks -----------------------*/

/* Call int 13h for HDD info. */

void get_max_tracks(void)
{
	regs.h.ah = drive_parms_function;
	regs.h.dl = drive;
	int86(disk, &regs, &regs);
	max_track = cx_to_track(regs.x.cx); 	/* get the coded maximum track */

	printf("\nLast track on drive No %x is .. %u - ", drive - 0x7f, max_track);

	no_of_drives = regs.h.dl;
}

/*-------------------------park_head---------------------------*/

void park_head(void)

{
	regs.h.ah = seek_function;
	regs.h.dl = drive;
	regs.x.cx = track_to_cx(max_track);

	int86(disk, &regs, &regs);

	printf("Parked at track %d\n", max_track);
}


/*
  
   The following 2 functions sort out the track (cylinder)
   number which is used by BIOS int 13h. The cylinder number is a
   10 bit value arranged in cx reg as follows:-

                    ch      cl

          cx     xxxxxxxx xx000000			     
                    ^     ^
                    |     | High order 2 bits of cylinder number
                    |
                    Low order 8 bits of cylinder number


*/

/*-------------------------cx_to_track--------------------------*/

/* Convert coded cx register value to a track number */


unsigned int cx_to_track(unsigned int cx_reg)
{
	return( ((cx_reg & mask1) << 2) | (cx_reg >>8) );
}


/*--------------------------track_to_cx--------------------------*/

/* Convert track no. to coded cx register value */


unsigned int track_to_cx(unsigned int track)
{
	return((track & mask3) >> 2)  |  (track << 8);
}

/* END */
